let dappaccountauth = {};

let { PrivateKey, PublicKey } = require("eosjs-ecc");
let helperaction = require("./common/helper");
const axios = require("axios").default;
let chaininfo = require("./common/chaininfo.json");
let sha256 = require("sha256");

dappaccountauth.registerdapp = async (
  env,
  email,
  contact,
  levelid,
  dappcontract,
  emailcode,
  contactcode
) => {
  try {
    if (!chaininfo[env]) {
      throw new Error("Must provide environment correctly : kylin / main");
    }
    if (!(parseInt(levelid) == 1 || parseInt(levelid) == 2)) {
      throw new Error("Level id must lie between 1 or 2");
    }
    let resregarr = [];
    let prvkeyarr = [];
    let typearr = [];
    for (let i = 0; i < parseInt(levelid); i++) {
      let prv_key = await PrivateKey.randomKey();
      prv_key = prv_key.toWif();
      prvkeyarr.push(prv_key);
      let pubkey = PrivateKey.fromString(prv_key).toPublic().toString();
      let respobj = {
        levtype: chaininfo[env].typeauthopt[i],
        typekey: i ? contact : email,
        otp: i ? contactcode : emailcode,
        pubkeylev1: pubkey,
      };
      resregarr.push(respobj);
      typearr.push(chaininfo[env].typeauthopt[i]);
    }

    const vacc = await helperaction.converttovacc(email);
    let regobj = {
      vaccount: vacc,
      email: email,
      contact: contact,
      levelid: levelid,
      levelinfo: resregarr,
      dappcontract: dappcontract,
    };
    console.log("data---", regobj);
    let resp = await helperaction.pushdappacc(
      env,
      chaininfo[env].contract,
      prvkeyarr[0],
      "regboth",
      regobj
    );
    console.log(typearr);
    let respkey = await axios.post(`${chaininfo[env].removeauthkey}`, {
      vaccount: vacc,
      authtype: typearr,
    });
    if (resp.message) {
      return {
        accname: vacc,
        key: prvkeyarr,
      };
    }
  } catch (err) {
    console.log(err);
    throw new Error(err);
  }
};

// dappaccountauth.registerdapp(
//   "kylin",
//   "rayarunima55@gmail.com",
//   "+918981301328",
//   1,
//   "dapptokencon",
//   "042634",
//   "064635"
// );

dappaccountauth.regcodereq = async (env, email, contact, levelid, dappname) => {
  try {
    if (!chaininfo[env]) {
      throw new Error("Must provide environment correctly : kylin / main");
    }
    if (!(parseInt(levelid) == 1 || parseInt(levelid) == 2)) {
      throw new Error("Level id must lie between 1 or 2");
    }
    const vacc = await helperaction.converttovacc(email);
    let resp = await axios.post(chaininfo[env].regreqcode, {
      vaccount: vacc,
      email: email.toLowerCase(),
      contact: contact,
      levelid: levelid,
      dappname: dappname
    });
    return resp.data;
  } catch (err) {
    console.log(err.response.data);
    throw new Error(err.response.data.message);
  }
};
//dappaccountauth.regcodereq('kylin', 'rayarunima55@gmail.com','+918981301328',2)

dappaccountauth.actauthcodereq = async (
  env,
  email,
  authtype,
  authkey,
  levelid,
  dappname
) => {
  try {
    if (!chaininfo[env]) {
      throw new Error("Must provide environment correctly : kylin / main");
    }
    if (!(parseInt(levelid) == 1 || parseInt(levelid) == 2)) {
      throw new Error("Level id must lie between 1 or 2");
    }
    const vacc = await helperaction.converttovacc(email.toLowerCase());
    let resp = await axios.post(chaininfo[env].authbydapp, {
      vaccount: vacc,
      authtype: authtype,
      authkey: authkey,
      levelid: levelid,
      dappname: dappname
    });
    console.log(resp.data);
    return resp.data;
  } catch (err) {
    console.log(err);
    throw new Error(err.response.data.message);
  }
};
// dappaccountauth.actauthcodereq(
//   "kylin",
//   "rayarunima55@gmail.com",
//   "email",
//   "rayarunima55@gmail.com",
//   1
// );
dappaccountauth.recoverdapp = async (
  env,
  email,
  dappcontract,
  auth_code,
  levelid,
  authkey,
) => {
  try {
    if (!chaininfo[env]) {
      throw new Error("Must provide environment correctly : kylin / main");
    }
    if (!(parseInt(levelid) == 1 || parseInt(levelid) == 2)) {
      throw new Error("Level id must lie between 1 or 2");
    }

    let prv_key = await PrivateKey.randomKey();
    prv_key = prv_key.toWif();
    console.log('----', prv_key)
    let pubkey = PrivateKey.fromString(prv_key).toPublic().toString();
    const vacc = await helperaction.converttovacc(email);
    let recoverobj = {
      vaccount: vacc,
      dappcontract: dappcontract,
      otp: auth_code,
      levelid: levelid,
      typeauth: chaininfo[env].typeauthopt[parseInt(levelid) - 1],
      dapppubkey: pubkey,
      authkey: authkey,
    };
    console.log("data---", recoverobj);
    let resp = await helperaction.pushdappacc(
      env,
      chaininfo[env].contract,
      prv_key,
      "dappreset",
      recoverobj
    );
    console.log(resp);
    let respkey = await axios.post(`${chaininfo[env].removeauthkey}`, {
      vaccount: vacc,
      authtype: [chaininfo[env].typeauthopt[parseInt(levelid) - 1]],
    });
    if (resp.message) {
      return {
        accname: vacc,
        key: prv_key,
      };
    }
  } catch (err) {
    console.log('--', err)
    throw new Error(err);
  }
};
//dappaccountauth.recoverdapp("kylin", "rayarunima55@gmail.com", "vaccconuser1", "199530", 1, "rayarunima55@gmail.com", "5KGeTxdh6FZwvfqVWPEBiBP3sFKEm1ud3pKjKmb4cpZ8Z4RNxoM")

dappaccountauth.removekey = async (env, dappcontract, vaccount_prvkey, email, levelid) => {
  try {
    if (!chaininfo[env]) {
      throw new Error("Must provide environment correctly : kylin / main");
    }
    if (!(parseInt(levelid) == 1 || parseInt(levelid) == 2)) {
      throw new Error("Level id must lie between 1 or 2");
    }
    const vaccount = await helperaction.converttovacc(email.toLowerCase());
    let delobj = {
      vaccount: vaccount,
      levelid: levelid,
      dappcontract: dappcontract,
    };
    console.log("data---", delobj);
    let resp = await helperaction.pushdappacc(
      env,
      chaininfo[env].contract,
      vaccount_prvkey,
      "dellevkey",
      delobj
    );
    console.log(resp);

    if (resp.message) {
      return {
        accname: vaccount,
        key: vaccount_prvkey,
      };
    }
  } catch (err) {
    throw new Error(err);
  }
};

//dappaccountauth.removekey("kylin", "vaccconuser1", "5Ht5DBrd3TerQCVCPEULPcGyXMXx3q1WNCC2eHgFHq5GkTtoiZk", "rayarunima55@gmail.com", 1)
//////////////////////////////
module.exports = dappaccountauth;
